package com.ims.eval.controller;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ims.common.utils.StringUtils;
import com.ims.eval.config.CustomException;
import com.ims.eval.entity.DeptAssessmentDeclaration;
import com.ims.eval.entity.DeptAssessmentDeclarationContent;
import com.ims.eval.entity.EvaluationDept;
import com.ims.eval.entity.dto.request.EmployeeDTO;
import com.ims.eval.entity.dto.request.SummaryInformationDTO;
import com.ims.eval.entity.dto.request.UserDTO;
import com.ims.eval.entity.dto.result.R;
import com.ims.eval.service.IDeptAssessmentDeclarationContentService;
import com.ims.eval.service.IDeptAssessmentDeclarationService;
import com.ims.eval.service.IEvaluationDeptService;
import com.ims.eval.service.IUserService;
import com.ims.eval.util.ExcelUtil;
import com.ims.eval.util.ExcelUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 绩效结果考核申报
 *
 * @author hlf
 * @date 2023/5/29 15:15
 * 文件说明：
 */
@Slf4j
@RestController
@RequestMapping("//evaluation-dept-ad")
public class DeptAssessmentDeclarationController {

	@Autowired
	private IDeptAssessmentDeclarationService deptAssessmentDeclarationService;

	@Autowired
	private IDeptAssessmentDeclarationContentService deptAssessmentDeclarationContentService;

	@Autowired
	private IEvaluationDeptService evaluationDeptService;

	@Autowired
	private IUserService userService;

	@Autowired
	private HttpServletRequest request;

	/**
	 * 绩效结果考核申报列表信息（分页）
	 *
	 * @param pageNum          当前记录起始索引
	 * @param pageSize         每页显示记录数
	 * @param orderNumber      单号
	 * @param deptName         部门名称
	 * @param annual           年度
	 * @param declarationMonth 申报月份
	 * @return 结果
	 */
	@GetMapping(value = "/list")
	public R list(
		@RequestParam(value = "pageNum") Integer pageNum,
		@RequestParam(value = "pageSize") Integer pageSize,
		@RequestParam(value = "deptId") String deptId,
		@RequestParam(value = "orderNumber", required = false) String orderNumber,
		@RequestParam(value = "deptName", required = false) String deptName,
		@RequestParam(value = "annual", required = false) String annual,
		@RequestParam(value = "declarationMonth", required = false) String declarationMonth) {
		IPage<DeptAssessmentDeclaration> list = deptAssessmentDeclarationService.listPage(pageNum, pageSize, deptId, orderNumber, deptName, annual, declarationMonth);
		return R.ok().data(list);
	}

	/**
	 * 绩效结果考核申报信息
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/listAll")
	public R listAll() {
		List<DeptAssessmentDeclaration> list = deptAssessmentDeclarationService.list();
		return R.ok().data(list);
	}

	/**
	 * 新增绩效结果考核申报信息
	 *
	 * @param deptAssessmentDeclaration 绩效结果考核申报实体
	 * @return 结果
	 */
	@PostMapping(value = "/save")
	public R addAll(@RequestBody DeptAssessmentDeclaration deptAssessmentDeclaration, HttpServletRequest request) {
		try {
			QueryWrapper<DeptAssessmentDeclaration> qw = new QueryWrapper<>();
			if (StringUtils.isNotEmpty(deptAssessmentDeclaration.getDeptId())) {
				qw.lambda().eq(DeptAssessmentDeclaration::getDeptId, deptAssessmentDeclaration.getDeptId());
			}
			if (StringUtils.isNotEmpty(deptAssessmentDeclaration.getAnnual())) {
				qw.lambda().eq(DeptAssessmentDeclaration::getAnnual, deptAssessmentDeclaration.getAnnual());
			}
			if (StringUtils.isNotEmpty(deptAssessmentDeclaration.getDeclarationMonth())) {
				qw.lambda().eq(DeptAssessmentDeclaration::getDeclarationMonth, deptAssessmentDeclaration.getDeclarationMonth());
			}
			DeptAssessmentDeclaration assessmentDeclaration = deptAssessmentDeclarationService.getOne(qw);
			if (ObjectUtil.isNotNull(assessmentDeclaration)) {
				return R.error(deptAssessmentDeclaration.getDeptName() + deptAssessmentDeclaration.getAnnual() + "年" + deptAssessmentDeclaration.getDeclarationMonth() + "月绩效结果考核申报已存在！");
			}
			boolean b = deptAssessmentDeclarationService.save(deptAssessmentDeclaration, request);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("保存失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 获取部门领导
	 *
	 * @return 结果
	 */
	@GetMapping(value = "/getDepartmentLeader/{id}")
	public R getDepartmentLeader(@PathVariable String id) {
		QueryWrapper<EvaluationDept> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().like(EvaluationDept::getDeptId, id);
		}
		EvaluationDept evaluationDept = evaluationDeptService.getOne(qw);
		return R.ok().data(evaluationDept);
	}

	/**
	 * 修改绩效结果考核申报信息
	 *
	 * @param deptAssessmentDeclaration 绩效结果考核申报实体
	 * @return 结果
	 */
	@PostMapping(value = "/modify")
	public R modify(@RequestBody DeptAssessmentDeclaration deptAssessmentDeclaration) {
		try {
			boolean b = deptAssessmentDeclarationService.updateById(deptAssessmentDeclaration);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("修改失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 批量删除绩效结果考核申报信息
	 *
	 * @param ids 主键s
	 * @return 结果
	 */
	@PostMapping(value = "/removeAll/{ids}")
	public R deleteAll(@PathVariable("ids") String ids) {
		try {
			String[] strings = ids.split(",");
			boolean b = deptAssessmentDeclarationService.removeByIds(Arrays.asList(strings));
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("删除失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 详情-头部信息
	 *
	 * @param id 业务主键
	 * @return 结果
	 */
	@GetMapping(value = "/detailsHead/{id}")
	public R detailsHead(@PathVariable("id") String id) {
		DeptAssessmentDeclaration deptAssessmentDeclaration = deptAssessmentDeclarationService.getById(id);
		return R.ok().data(deptAssessmentDeclaration);
	}

	/**
	 * 详情
	 *
	 * @param id 业务主键
	 * @return 结果
	 */
	@GetMapping(value = "/details/{id}")
	public R details(@PathVariable("id") String id) {
		List<DeptAssessmentDeclarationContent> list = deptAssessmentDeclarationContentService.detailsList(id);
		return R.ok().data(list);
	}

	/**
	 * 详情-修改/新增
	 *
	 * @param deptAssessmentDeclarationContentList 修改内容
	 * @return 结果
	 */
	@PostMapping(value = "/update")
	public R update(@RequestBody List<DeptAssessmentDeclarationContent> deptAssessmentDeclarationContentList) {
		try {
			boolean b = deptAssessmentDeclarationContentService.saveOrUpdateBatch(deptAssessmentDeclarationContentList);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("新增/修改失败，人员重复！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 详情-批量删除
	 *
	 * @param ids 主键
	 * @return 结果
	 */
	@GetMapping(value = "/delete/{ids}")
	public R delete(@PathVariable String ids) {
		try {
			String[] strings = ids.split(",");
			boolean b = deptAssessmentDeclarationContentService.removeByIds(Arrays.asList(strings));
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("删除失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 导入
	 *
	 * @param file 文件
	 * @return 结果
	 */
	@PostMapping(value = "/import")
	public R importData(@RequestParam("file") MultipartFile file, @RequestParam("id") String id) {
		if (!file.isEmpty()) {
			try {
				//成功条数
				int successCount = 0;
				//更新条数
				int renewalCount = 0;
				//失败条数
				int loseCount = 0;
				//获取原始的文件名
				String originalFilename = file.getOriginalFilename();
				//获取文件类型
				String fileType = originalFilename.substring(originalFilename.lastIndexOf(".") + 1, originalFilename.length());
				//默认从第一行开始读取
				int startRows = 1;
				//获取输入流
				InputStream is = file.getInputStream();
				//Excel导入导出的单元类
				List<String[]> strings = ExcelUtil.readData(fileType, startRows, true, is);
				//遍历Excel表每一行的数据
				for (String[] str : strings) {
					DeptAssessmentDeclarationContent deptAssessmentDeclarationContent = new DeptAssessmentDeclarationContent();
					deptAssessmentDeclarationContent.setAssessmentDeclarationId(id);
					deptAssessmentDeclarationContent.setEmployeeNo(str[0]);
					JSONObject jsonArr = userService.pageList(1, 500, "", "", "", str[0], "", "", request);
					JSONObject jsonArr1 = (JSONObject) jsonArr.get("data");
					JSONArray array = JSONUtil.parseArray(jsonArr1.get("records"));
					List<UserDTO> userList = JSONUtil.toList(array, UserDTO.class);
					if (userList.size() > 0) {
						deptAssessmentDeclarationContent.setEmployeeId(userList.get(0).getId());
						deptAssessmentDeclarationContent.setEmployeeName(userList.get(0).getName());
					}
					deptAssessmentDeclarationContent.setSuggestedValue(str[2]);
					deptAssessmentDeclarationContent.setSerialNumber(Convert.toInt(str[3]));
					QueryWrapper<DeptAssessmentDeclarationContent> qw = new QueryWrapper<>();
					if (StringUtils.isNotEmpty(deptAssessmentDeclarationContent.getAssessmentDeclarationId())) {
						qw.lambda().like(DeptAssessmentDeclarationContent::getAssessmentDeclarationId, deptAssessmentDeclarationContent.getAssessmentDeclarationId());
					}
					if (StringUtils.isNotEmpty(deptAssessmentDeclarationContent.getEmployeeNo())) {
						qw.lambda().like(DeptAssessmentDeclarationContent::getEmployeeNo, deptAssessmentDeclarationContent.getEmployeeNo());
					}
					if (StringUtils.isNotEmpty(deptAssessmentDeclarationContent.getEmployeeId())) {
						qw.lambda().like(DeptAssessmentDeclarationContent::getEmployeeId, deptAssessmentDeclarationContent.getEmployeeId());
					}
					List<DeptAssessmentDeclarationContent> objList = deptAssessmentDeclarationContentService.list(qw);
					if (objList.size() == 1) {//修改
						deptAssessmentDeclarationContent.setId(objList.get(0).getId());
						deptAssessmentDeclarationContentService.updateById(deptAssessmentDeclarationContent);
						renewalCount++;
					} else if (objList.size() == 0) {//新增
						deptAssessmentDeclarationContentService.save(deptAssessmentDeclarationContent);
						successCount++;
					} else {
						loseCount++;
					}
				}
				String msg = "导入成功，本次新增" + successCount + "条，更新" + renewalCount + "条，失败" + loseCount + "条数据。";
				return R.ok().message(msg);
			} catch (Exception e) {
				log.error("错误", e);
				return R.customError(e.getMessage()).data("失败！");
			}
		}
		return R.error("上传文件为空！");
	}

	/**
	 * 修改状态
	 *
	 * @param id    主键
	 * @param stage 状态
	 * @return 结果
	 */
	@PostMapping(value = "/targetStart")
	public R targetStart(@RequestParam(value = "id") String id,
						 @RequestParam(value = "stage") String stage) {
		try {
			DeptAssessmentDeclaration deptAssessmentDeclaration = deptAssessmentDeclarationService.getById(id);
			deptAssessmentDeclaration.setStage(stage);
			boolean b = deptAssessmentDeclarationService.updateById(deptAssessmentDeclaration);
			if (b) {
				return R.ok().data(b);
			} else {
				return R.error("失败！");
			}
		} catch (CustomException e) {
			return R.customError(e.getMessage()).data("失败！");
		}
	}

	/**
	 * 生成报表
	 *
	 * @param id       主键
	 * @param response response
	 * @throws Exception 异常
	 */
	@GetMapping(value = "/generateReport/{id}")
	public void generateReport(@PathVariable("id") String id, HttpServletResponse response) throws Exception {
		response.setContentType("application/vnd.ms-excel;charset=UTF-8");
		response.setHeader("Content-disposition", "attachment; filename=".concat(URLEncoder.encode("公司本部部门及员工月度绩效考核结果申报表.xlsx", "UTF-8")));
		response.setHeader("Cache-Control", "no-cache, no-store, must-revalidate");
		response.setHeader("Pragma", "no-cache");
		response.setDateHeader("Expires", 0);
		List<EmployeeDTO> employeeDTOList = new ArrayList<>();
		// 添加员工信息
		QueryWrapper<DeptAssessmentDeclarationContent> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().like(DeptAssessmentDeclarationContent::getAssessmentDeclarationId, id);
		}
		List<DeptAssessmentDeclarationContent> deptAssessmentDeclarationContentList = deptAssessmentDeclarationContentService.list(qw);
		for (DeptAssessmentDeclarationContent deptAssessmentDeclarationContent : deptAssessmentDeclarationContentList) {
			EmployeeDTO employeeDTO = new EmployeeDTO();
			employeeDTO.setName(deptAssessmentDeclarationContent.getEmployeeName());
			employeeDTO.setCoefficient(deptAssessmentDeclarationContent.getSuggestedValue());
			employeeDTOList.add(employeeDTO);
		}
		DeptAssessmentDeclaration deptAssessmentDeclaration = deptAssessmentDeclarationService.getById(id);
		String departmentName = deptAssessmentDeclaration.getDeptName();
		String month = deptAssessmentDeclaration.getDeclarationMonth();
		String level = deptAssessmentDeclaration.getRatingGrade();
		String leader = deptAssessmentDeclaration.getDeptLeaderName();
		String declarationReason = deptAssessmentDeclaration.getDeclarationReason();
		ExcelUtils.createExcel(employeeDTOList, departmentName, month, level, leader, declarationReason, response.getOutputStream());
	}

	/**
	 * 统计汇总数据
	 *
	 * @param annual 年度
	 * @param month  月份
	 */
	@GetMapping(value = "/statisticalSummary/{annual}/{month}")
	public R statisticalSummary(@PathVariable("annual") String annual, @PathVariable("month") String month) {
		List<SummaryInformationDTO> summaryInformationDTOList = new ArrayList<>();
		QueryWrapper<DeptAssessmentDeclaration> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(annual)) {
			qw.lambda().like(DeptAssessmentDeclaration::getAnnual, annual);
		}
		if (StringUtils.isNotEmpty(month)) {
			qw.lambda().like(DeptAssessmentDeclaration::getDeclarationMonth, month);
		}
		List<DeptAssessmentDeclaration> deptAssessmentDeclarationList = deptAssessmentDeclarationService.list(qw);
		for (DeptAssessmentDeclaration deptAssessmentDeclaration : deptAssessmentDeclarationList) {
			EvaluationDept evaluationDept = evaluationDeptService.getById(deptAssessmentDeclaration.getDeptId());
			List<DeptAssessmentDeclarationContent> deptAssessmentDeclarationContentList = deptAssessmentDeclarationContentService.detailsList(deptAssessmentDeclaration.getId());
			for (DeptAssessmentDeclarationContent deptAssessmentDeclarationContent : deptAssessmentDeclarationContentList) {
				SummaryInformationDTO summaryInformationDTO = new SummaryInformationDTO();
				summaryInformationDTO.setEmployeeNo(deptAssessmentDeclarationContent.getEmployeeNo());
				summaryInformationDTO.setEmployeeName(deptAssessmentDeclarationContent.getEmployeeName());
				summaryInformationDTO.setSuggestedValue(deptAssessmentDeclarationContent.getSuggestedValue());
				summaryInformationDTO.setReportingDepartment(deptAssessmentDeclaration.getDeptName());
				summaryInformationDTO.setDeclarationLevel(deptAssessmentDeclaration.getRatingGrade());
				if (null != evaluationDept) {
					summaryInformationDTO.setSerialNumber(evaluationDept.getSerialNumber());
				}
				summaryInformationDTOList.add(summaryInformationDTO);
			}
		}
		List<SummaryInformationDTO> list = new ArrayList<>();
		Map<String, List<SummaryInformationDTO>> map = summaryInformationDTOList.stream().collect(Collectors.groupingBy(SummaryInformationDTO::getEmployeeNo));
		map.forEach((key, value) -> {
			for (SummaryInformationDTO summaryInformationDTO : value) {
				summaryInformationDTO.setDeclarationsNumber(String.valueOf(value.size()));
			}
			list.addAll(value);
		});
		list.stream().sorted(Comparator.comparing(SummaryInformationDTO::getSerialNumber, Comparator.nullsLast(Integer::compareTo))).collect(Collectors.toList());
		return R.ok().data(list);
	}

	/**
	 * 生成统计汇总报表
	 *
	 * @param annual   年度
	 * @param month    月份
	 * @param response response
	 * @throws Exception 异常
	 */
	@GetMapping(value = "/generateStatisticalSummaryReports/{annual}/{month}")
	public void generateStatisticalSummaryReports(@PathVariable("annual") String annual, @PathVariable("month") String month, HttpServletResponse response) throws Exception {
		response.setContentType("application/vnd.ms-excel;charset=UTF-8");
		response.setHeader("Content-disposition", "attachment; filename=".concat(URLEncoder.encode("绩效结果考核申报汇总表.xlsx", "UTF-8")));
		response.setHeader("Cache-Control", "no-cache, no-store, must-revalidate");
		response.setHeader("Pragma", "no-cache");
		response.setDateHeader("Expires", 0);
		List<SummaryInformationDTO> summaryInformationDTOList = new ArrayList<>();
		QueryWrapper<DeptAssessmentDeclaration> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(annual)) {
			qw.lambda().like(DeptAssessmentDeclaration::getAnnual, annual);
		}
		if (StringUtils.isNotEmpty(month)) {
			qw.lambda().like(DeptAssessmentDeclaration::getDeclarationMonth, month);
		}
		List<DeptAssessmentDeclaration> deptAssessmentDeclarationList = deptAssessmentDeclarationService.list(qw);
		for (DeptAssessmentDeclaration deptAssessmentDeclaration : deptAssessmentDeclarationList) {
			List<DeptAssessmentDeclarationContent> deptAssessmentDeclarationContentList = deptAssessmentDeclarationContentService.detailsList(deptAssessmentDeclaration.getId());
			for (DeptAssessmentDeclarationContent deptAssessmentDeclarationContent : deptAssessmentDeclarationContentList) {
				SummaryInformationDTO summaryInformationDTO = new SummaryInformationDTO();
				summaryInformationDTO.setEmployeeNo(deptAssessmentDeclarationContent.getEmployeeNo());
				summaryInformationDTO.setEmployeeName(deptAssessmentDeclarationContent.getEmployeeName());
				summaryInformationDTO.setSuggestedValue(deptAssessmentDeclarationContent.getSuggestedValue());
				summaryInformationDTO.setReportingDepartment(deptAssessmentDeclaration.getDeptName());
				summaryInformationDTO.setDeclarationLevel(deptAssessmentDeclaration.getRatingGrade());
				summaryInformationDTOList.add(summaryInformationDTO);
			}
		}
		List<SummaryInformationDTO> list = new ArrayList<>();
		Map<String, List<SummaryInformationDTO>> map = summaryInformationDTOList.stream().collect(Collectors.groupingBy(SummaryInformationDTO::getEmployeeNo));
		map.forEach((key, value) -> {
			for (SummaryInformationDTO summaryInformationDTO : value) {
				summaryInformationDTO.setDeclarationsNumber(String.valueOf(value.size()));
			}
			list.addAll(value);
		});
		list.stream().sorted(Comparator.comparing(SummaryInformationDTO::getSerialNumber, Comparator.nullsLast(Integer::compareTo))).collect(Collectors.toList());
		ExcelUtils.createCollectExcel(list, annual, month, response.getOutputStream());
	}

	/**
	 * 参与考核人数
	 *
	 * @param id 主键
	 * @return 结果
	 */
	@GetMapping(value = "/participantsNumber/{id}")
	public R participantsNumber(@PathVariable String id) {
		QueryWrapper<DeptAssessmentDeclarationContent> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(id)) {
			qw.lambda().like(DeptAssessmentDeclarationContent::getAssessmentDeclarationId, id);
		}
		int count = deptAssessmentDeclarationContentService.list(qw).size();
		return R.ok().data(count);
	}
}
