package com.ims.eval.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ims.common.utils.StringUtils;
import com.ims.eval.dao.DeptAssessmentDeclarationMapper;
import com.ims.eval.entity.DeptAssessmentDeclaration;
import com.ims.eval.entity.DeptAssessmentDeclarationContent;
import com.ims.eval.entity.EvaluationDept;
import com.ims.eval.entity.EvaluationDeptRating;
import com.ims.eval.entity.dto.request.UserDTO;
import com.ims.eval.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

/**
 * @author hlf
 * @date 2023/5/29 15:01
 * 文件说明：
 */
@Service
public class DeptAssessmentDeclarationServiceImpl extends ServiceImpl<DeptAssessmentDeclarationMapper, DeptAssessmentDeclaration> implements IDeptAssessmentDeclarationService {

	@Autowired
	private IDeptAssessmentDeclarationContentService deptAssessmentDeclarationContentService;

	@Autowired
	private IUserService userService;

	@Autowired
	private IEvaluationDeptService evaluationDeptService;

	@Autowired
	private IEvaluationDeptRatingService evaluationDeptRatingService;

	@Override
	public IPage<DeptAssessmentDeclaration> listPage(Integer pageNum, Integer pageSize, String deptId, String orderNumber, String deptName, String annual, String declarationMonth) {
		if ("23031009".equals(deptId)) {//组织人事部
			deptId = "";
		}
		if (null == pageNum || null == pageSize) {
			throw new RuntimeException("分页参数为空");
		}
		//构造分页构造器
		Page<DeptAssessmentDeclaration> page = new Page<>(pageNum, pageSize);
		return baseMapper.selectListPage(page, deptId, orderNumber, deptName, annual, declarationMonth);
	}

	@Override
	public boolean save(DeptAssessmentDeclaration deptAssessmentDeclaration, HttpServletRequest request) {
		QueryWrapper<DeptAssessmentDeclaration> qw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(deptAssessmentDeclaration.getAnnual())) {
			qw.lambda().like(DeptAssessmentDeclaration::getAnnual, deptAssessmentDeclaration.getAnnual());
		}
		if (StringUtils.isNotEmpty(deptAssessmentDeclaration.getDeclarationMonth())) {
			qw.lambda().eq(DeptAssessmentDeclaration::getDeclarationMonth, deptAssessmentDeclaration.getDeclarationMonth());
		}
		int count = super.count(qw);
		String id = Convert.toStr(IdUtil.getSnowflake(1, 1).nextId());
		deptAssessmentDeclaration.setId(id);
		deptAssessmentDeclaration.setOrderNumber("KHSB_GDDL_" + deptAssessmentDeclaration.getAnnual() + "_" + StringUtils.addZeroForStr(String.valueOf(Integer.parseInt(deptAssessmentDeclaration.getDeclarationMonth())), 2, "l") + StringUtils.addZeroForStr(String.valueOf(count + 1), 2, "l"));
		deptAssessmentDeclaration.setRatingGrade(deptAssessmentDeclaration.getDeclarationLevel());
		deptAssessmentDeclaration.setStage("流程未启动");
		deptAssessmentDeclaration.setCreateTime(DateUtil.date());
		List<DeptAssessmentDeclarationContent> deptAssessmentDeclarationContentList = new ArrayList<>();
		QueryWrapper<EvaluationDept> deptQw = new QueryWrapper<>();
		if (StringUtils.isNotEmpty(deptAssessmentDeclaration.getDeptId())) {
			deptQw.lambda().eq(EvaluationDept::getDeptId, deptAssessmentDeclaration.getDeptId());
		}
		EvaluationDept evaluationDept = evaluationDeptService.getOne(deptQw);
		if (null != evaluationDept) {
			if ("".equals(evaluationDept.getParentId()) || null == evaluationDept.getParentId()) {//父级部门为空 表示当前为父级
				List<UserDTO> secondaryList = new ArrayList<>();
				QueryWrapper<EvaluationDept> secondaryDeptQw = new QueryWrapper<>();
				if (StringUtils.isNotEmpty(evaluationDept.getDeptId())) {
					secondaryDeptQw.lambda().eq(EvaluationDept::getParentId, evaluationDept.getDeptId());
				}
				List<EvaluationDept> evaluationDeptList = evaluationDeptService.list(secondaryDeptQw);
				for (EvaluationDept obj : evaluationDeptList) {
					JSONObject jsonArr = userService.pageList(1, 500, obj.getDeptId(), "", "", "", "", "", request);
					JSONObject jsonArr1 = (JSONObject) jsonArr.get("data");
					JSONArray array = JSONUtil.parseArray(jsonArr1.get("records"));
					List<UserDTO> userList = JSONUtil.toList(array, UserDTO.class);
					secondaryList.addAll(userList);
				}
				JSONObject jsonArr = userService.pageList(1, 500, deptAssessmentDeclaration.getDeptId(), "", "", "", "", "", request);
				JSONObject jsonArr1 = (JSONObject) jsonArr.get("data");
				JSONArray array = JSONUtil.parseArray(jsonArr1.get("records"));
				List<UserDTO> userList = JSONUtil.toList(array, UserDTO.class);
				userList.removeAll(secondaryList);
				for (int i = 0; i < userList.size(); i++) {
					if (StringUtils.isNotEmpty(userList.get(i).getNo())) {
						DeptAssessmentDeclarationContent deptAssessmentDeclarationContent = new DeptAssessmentDeclarationContent();
						deptAssessmentDeclarationContent.setAssessmentDeclarationId(id);
						deptAssessmentDeclarationContent.setSerialNumber(i + 1);
						deptAssessmentDeclarationContent.setEmployeeId(userList.get(i).getId());
						deptAssessmentDeclarationContent.setEmployeeNo(userList.get(i).getNo());
						deptAssessmentDeclarationContent.setEmployeeName(userList.get(i).getName());
						deptAssessmentDeclarationContent.setSuggestedValue("1");
						deptAssessmentDeclarationContentList.add(deptAssessmentDeclarationContent);
					}
				}
			} else {//子级部门
				JSONObject jsonArr = userService.pageList(1, 500, deptAssessmentDeclaration.getDeptId(), "", "", "", "", "", request);
				JSONObject jsonArr1 = (JSONObject) jsonArr.get("data");
				JSONArray array = JSONUtil.parseArray(jsonArr1.get("records"));
				List<UserDTO> userList = JSONUtil.toList(array, UserDTO.class);
				for (int i = 0; i < userList.size(); i++) {
					if (StringUtils.isNotEmpty(userList.get(i).getNo())) {
						DeptAssessmentDeclarationContent deptAssessmentDeclarationContent = new DeptAssessmentDeclarationContent();
						deptAssessmentDeclarationContent.setAssessmentDeclarationId(id);
						deptAssessmentDeclarationContent.setSerialNumber(i + 1);
						deptAssessmentDeclarationContent.setEmployeeId(userList.get(i).getId());
						deptAssessmentDeclarationContent.setEmployeeNo(userList.get(i).getNo());
						deptAssessmentDeclarationContent.setEmployeeName(userList.get(i).getName());
						deptAssessmentDeclarationContent.setSuggestedValue("1");
						deptAssessmentDeclarationContentList.add(deptAssessmentDeclarationContent);
					}
				}
			}
		}
		boolean b = super.save(deptAssessmentDeclaration);
		if (b) {
			b = deptAssessmentDeclarationContentService.saveBatch(deptAssessmentDeclarationContentList);
		}
		return b;
	}

	@Override
	public boolean removeByIds(List<String> ids) {
		boolean b = false;
		for (String id : ids) {
			DeptAssessmentDeclaration deptAssessmentDeclaration = super.getById(id);
			if ("流程未启动".equals(deptAssessmentDeclaration.getStage()) || "流程已结束".equals(deptAssessmentDeclaration.getStage())) {
				deptAssessmentDeclarationContentService.removeByAssessmentDeclarationId(id);
				super.removeById(id);
				b = true;
			} else {
				return b;
			}
		}
		return b;
	}

	@Override
	public boolean editState(String id, String state, String instId) {
		DeptAssessmentDeclaration deptAssessmentDeclaration = baseMapper.selectById(id);
		if (StringUtils.isNotEmpty(state)) {
			deptAssessmentDeclaration.setStage(state);
		}
		if (StringUtils.isNotEmpty(instId)) {
			deptAssessmentDeclaration.setInstId(instId);
		}
		super.saveOrUpdate(deptAssessmentDeclaration);
		if ("流程已结束".equals(deptAssessmentDeclaration.getStage())) {
			QueryWrapper<EvaluationDeptRating> qw = new QueryWrapper<>();
			if (StringUtils.isNotEmpty(deptAssessmentDeclaration.getDeptId())) {
				qw.lambda().eq(EvaluationDeptRating::getDeptId, deptAssessmentDeclaration.getDeptId());
			}
			if (StringUtils.isNotEmpty(deptAssessmentDeclaration.getAnnual())) {
				qw.lambda().eq(EvaluationDeptRating::getAnnual, deptAssessmentDeclaration.getAnnual());
			}
			EvaluationDeptRating evaluationDeptRating = evaluationDeptRatingService.getOne(qw);
			if (null != evaluationDeptRating) {
				insertContent(deptAssessmentDeclaration, evaluationDeptRating);
				evaluationDeptRatingService.updateById(evaluationDeptRating);
			} else {
				EvaluationDeptRating obj = new EvaluationDeptRating();
				insertContent(deptAssessmentDeclaration, obj);
				evaluationDeptRatingService.save(obj);
			}
		}
		return true;
	}

	private static void insertContent(DeptAssessmentDeclaration deptAssessmentDeclaration, EvaluationDeptRating evaluationDeptRating) {
		if ("1".equals(deptAssessmentDeclaration.getDeclarationMonth())) {
			evaluationDeptRating.setJan(deptAssessmentDeclaration.getRatingGrade());
			evaluationDeptRating.setJanContent(deptAssessmentDeclaration.getDeclarationReason());
		}
		if ("2".equals(deptAssessmentDeclaration.getDeclarationMonth())) {
			evaluationDeptRating.setFeb(deptAssessmentDeclaration.getRatingGrade());
			evaluationDeptRating.setFebContent(deptAssessmentDeclaration.getDeclarationReason());
		}
		if ("3".equals(deptAssessmentDeclaration.getDeclarationMonth())) {
			evaluationDeptRating.setMar(deptAssessmentDeclaration.getRatingGrade());
			evaluationDeptRating.setMarContent(deptAssessmentDeclaration.getDeclarationReason());
		}
		if ("4".equals(deptAssessmentDeclaration.getDeclarationMonth())) {
			evaluationDeptRating.setApr(deptAssessmentDeclaration.getRatingGrade());
			evaluationDeptRating.setAprContent(deptAssessmentDeclaration.getDeclarationReason());
		}
		if ("5".equals(deptAssessmentDeclaration.getDeclarationMonth())) {
			evaluationDeptRating.setMay(deptAssessmentDeclaration.getRatingGrade());
			evaluationDeptRating.setMayContent(deptAssessmentDeclaration.getDeclarationReason());
		}
		if ("6".equals(deptAssessmentDeclaration.getDeclarationMonth())) {
			evaluationDeptRating.setJun(deptAssessmentDeclaration.getRatingGrade());
			evaluationDeptRating.setJunContent(deptAssessmentDeclaration.getDeclarationReason());
		}
		if ("7".equals(deptAssessmentDeclaration.getDeclarationMonth())) {
			evaluationDeptRating.setJul(deptAssessmentDeclaration.getRatingGrade());
			evaluationDeptRating.setJulContent(deptAssessmentDeclaration.getDeclarationReason());
		}
		if ("8".equals(deptAssessmentDeclaration.getDeclarationMonth())) {
			evaluationDeptRating.setAug(deptAssessmentDeclaration.getRatingGrade());
			evaluationDeptRating.setAugContent(deptAssessmentDeclaration.getDeclarationReason());
		}
		if ("9".equals(deptAssessmentDeclaration.getDeclarationMonth())) {
			evaluationDeptRating.setSep(deptAssessmentDeclaration.getRatingGrade());
			evaluationDeptRating.setSepContent(deptAssessmentDeclaration.getDeclarationReason());
		}
		if ("10".equals(deptAssessmentDeclaration.getDeclarationMonth())) {
			evaluationDeptRating.setOct(deptAssessmentDeclaration.getRatingGrade());
			evaluationDeptRating.setOctContent(deptAssessmentDeclaration.getDeclarationReason());
		}
		if ("11".equals(deptAssessmentDeclaration.getDeclarationMonth())) {
			evaluationDeptRating.setNov(deptAssessmentDeclaration.getRatingGrade());
			evaluationDeptRating.setNovContent(deptAssessmentDeclaration.getDeclarationReason());
		}
		if ("12".equals(deptAssessmentDeclaration.getDeclarationMonth())) {
			evaluationDeptRating.setDece(deptAssessmentDeclaration.getRatingGrade());
			evaluationDeptRating.setDeceContent(deptAssessmentDeclaration.getDeclarationReason());
		}
	}
}
